###########################################################################
## R script belonging to the tutorial 'Regression modelling for digital soil mapping'
## Version: 1.4, 20 March 2017
## Author: Dr. B. Kempen
## Copyright (CC-BY-NC): ISRIC - World Soil Information
###########################################################################


## ----echo=FALSE, results='hide'------------------------------------------
# CHANGE PATH TO LOCAL PATH (the location of the '2_Regression_Modelling' folder)
setwd("D:/ISRIC/Training/Modules/Regression modelling in R/Workingdir/")


###########################################################################
##### 1 Initialization

## ----warning=FALSE-------------------------------------------------------
# load libraries
require(GSIF)
require(plotKML)
require(sp)
require(gstat)
require(rgdal)
require(aqp)
require(randomForest)
require(plyr)
require(ggplot2)
require(e1071)
require(leaflet)
library(htmlwidgets)
library(RColorBrewer)
library(RSAGA)

# set random seed
set.seed(170317)

## ------------------------------------------------------------------------
grs80 <- "+proj=longlat +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +no_defs"
wgs84 <- "+proj=longlat +datum=WGS84  +no_defs +ellps=WGS84  +towgs84=0,0,0"
gda94 <- "+proj=utm +zone=55 +south +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"


###########################################################################
##### 2 Soil data exploration and preprocessing

## ------------------------------------------------------------------------
data(edgeroi)

## ------------------------------------------------------------------------
str(edgeroi)

head(edgeroi[[1]][,1:4])
head(edgeroi[[2]])

## ------------------------------------------------------------------------
# copy the horizon information to a new object
edgeroi.spc <-  edgeroi[[2]]

# convert to SoilProfileCollection class
depths(edgeroi.spc) <- SOURCEID ~ UHDICM + LHDICM

# inspect result
str(edgeroi.spc)

## ------------------------------------------------------------------------
# add site data to SoilProfileCollection object
site(edgeroi.spc) <- join(x = edgeroi.spc@site, y = edgeroi[[1]], by = "SOURCEID") 

# add coordinates to SoilProfileCollection object
coordinates(edgeroi.spc) <- ~LONGDA94+LATGDA94
proj4string(edgeroi.spc) <- CRS(grs80)

# inspect the result
str(edgeroi.spc)

## ----results='hide'------------------------------------------------------
mps <- mpspline(edgeroi.spc, var.name="CLYPPT", vlow=0, vhigh=100)

## ------------------------------------------------------------------------
str(mps)

## ------------------------------------------------------------------------
mps$idcol <- edgeroi.spc@site$SOURCEID

## ------------------------------------------------------------------------
d <- data.frame(
  id = mps$idcol,
  x = edgeroi.spc@sp@coords[,1],
  y = edgeroi.spc@sp@coords[,2],
  z = mps$var.std[,2]
)

## ------------------------------------------------------------------------
d$z <- round(d$z, digits = 1)

## ------------------------------------------------------------------------
load("edgeroi.grids.rda")

## ------------------------------------------------------------------------
# create spatial objects
coordinates(d) <- ~x+y

# get the projection
proj4string(d) # no projection set

# set projection
proj4string(d) <- CRS(grs80)
proj4string(d)

## ------------------------------------------------------------------------
# reproject to GDA94 system
d <- spTransform(d, CRSobj = CRS(gda94))

## ------------------------------------------------------------------------
# convert grid data to spatial object
gridded(edgeroi.grids) <- ~x+y
proj4string(edgeroi.grids) <- CRS(gda94)

## ------------------------------------------------------------------------
# overlay
dum <- over(x = d, y = edgeroi.grids)

# append covariate data to point data
d@data <- cbind(d@data, dum)

# clean-up
rm(dum)

## ----results='hide'------------------------------------------------------
# identify points with similar coordinates (gives problems with spatial modelling)
zerodist(d, zero = 0.0, unique.ID = FALSE)

# remove duplicates
d <- remove.duplicates(d, zero = 0.0, remove.second = TRUE)

# convert to data.frame
d <- as(d, Class="data.frame")

# remove observations without covariate data, clay data or coordinates
d <- na.omit(d)


###########################################################################
##### 3 Linear Regression Modelling

## ----fig.width=12,fig.height=6-------------------------------------------
# create ggplot object
gg.z <- ggplot()+
  geom_point(
    data = d,
    mapping = aes(x = x * 1.0e-3, y = y * 1.0e-3, colour = z),
    size = 3
  ) +
  scale_colour_continuous(
    name = "Clay content (%)",
    low = "#99CCFF",
    high = "#003333"
  ) +
  scale_x_continuous(name = "Easting (km)") +
  scale_y_continuous(name = "Northing (km)") +
  coord_equal(ratio = 1)

# plot
gg.z

## ----out.width='6cm', out.height='6cm', fig.show='hold'------------------
# summary statistics
summary(d$z)

# skewness statistics (e1071 package)
skewness(d$z)

# histogram
hist(d$z, col = "lightblue")

# qqplot
qqnorm(d$z)
qqline(d$z)

## ------------------------------------------------------------------------
# define trend model
trend <- z~MVBSRT6+TI1LAN6+TI2LAN6+PCKGAD6+RUTGAD6+PCTGAD6+DEMSRT5+TWISRT5+PMTGEO5+EV1MOD5+EV2MOD5+EV3MOD5
class(trend)

# or alternatively and more generic
names(d)
trend <- as.formula(paste("z~",paste(names(d)[3:14],collapse="+")))
trend

## ------------------------------------------------------------------------
# fit lm model
z.lm <- lm(trend, data=d)

# inspect model
summary(z.lm)

## ----results='hide'------------------------------------------------------
# stepwise selection of covariates 
z.slm <- step(z.lm)

## ------------------------------------------------------------------------
# inspect model
summary(z.slm)

## ----fig.width=8, fig.height=5-------------------------------------------
plot(z.slm$fitted.values, d$z, xlab = "Fitted (%)", ylab = "Observed (%)")
abline(0,1)

## ----out.width='4cm', out.height='4cm', fig.show='hold'------------------
# summary statistics
summary(resid(z.slm))

# skewness statistics (e1071 package)
skewness(resid(z.slm))

# histogram
hist(resid(z.slm), col = "lightblue")

# qqplot
qqnorm(resid(z.slm))
qqline(resid(z.slm))

# residual plot
plot(z.slm$fitted.values, resid(z.slm), xlab = "Fitted (%)", ylab = "Residual (%)")
abline(0,0)

## ------------------------------------------------------------------------
# predict with standard errors
p <- predict.lm(z.slm, data = d, se.fit=TRUE)
str(p)

## ------------------------------------------------------------------------
# 95% confidence interval of the mean (expected value)
p$ci.l <- p$fit - 1.96*p$se.fit; p$ci.l <- unname(p$ci.l) # lower limit
p$ci.u <- p$fit + 1.96*p$se.fit; p$ci.u <- unname(p$ci.u) # upper limit
str(p)

## ------------------------------------------------------------------------
# compute the prediction standard deviation
u <- sqrt(p$se.fit**2+p$residual.scale**2)
head(round(u,3))

# 90% prediction interval
p$pi.l <- p$fit - 1.645*u; p$pi.l <- unname(p$pi.l)
p$pi.u <- p$fit + 1.645*u; p$pi.u <- unname(p$pi.u)
str(p)

## ------------------------------------------------------------------------
# define selected trend model (based on stepwise selection)
trend.sw <- z~MVBSRT6+TI1LAN6+TI2LAN6+PCKGAD6+RUTGAD6+PCTGAD6+DEMSRT5+PMTGEO5+EV2MOD5+EV3MOD5

## ----out.width='5cm', out.height='5cm'-----------------------------------
# convert data.frame to a spatial object
coordinates(d) <- ~x+y

# set coordinate projection
proj4string(d) <- CRS(gda94)

# compute sample variogram from the trend residuals
v <- variogram(trend.sw, data = d, width = 1000)
plot(v)

## ----out.width='5cm', out.height='5cm'-----------------------------------
# define variogram model: initial values for psill, range, nugget based on sample variogram
vm <- vgm(psill = 30, model = "Sph", range = 5000, nugget = 80)

# fit variogram model to the experimental variogram
vmf <- fit.variogram(v, model = vm)

# plot variograms
plot(v,vmf)

## ------------------------------------------------------------------------
p.tr <- predict(z.slm, newdata = edgeroi.grids)

## ------------------------------------------------------------------------
d$resid <- resid(z.slm)

## ----results='hide'------------------------------------------------------
rk <- krige(resid ~ 1, locations = d, newdata = edgeroi.grids, model = vmf, debug.level = -1)

## ------------------------------------------------------------------------
str(rk)

## ------------------------------------------------------------------------
rk$trend <- p.tr
rk$pred <- rk$trend + rk$var1.pred

## ------------------------------------------------------------------------
rk$lower <- rk$pred - 1.645 * sqrt(rk$var1.var)
rk$upper <- rk$pred + 1.645 * sqrt(rk$var1.var)

# summarize predictions
summary(rk@data)

# summary regression-kriging standard deviation
summary(sqrt(rk$var1.var))

## ------------------------------------------------------------------------
# correct predictions
rk$pred <- ifelse(test = rk$pred<0, yes = 0, no = rk$pred)
rk$lower <- ifelse(test = rk$lower<0, yes = 0, no = rk$lower)
rk$upper <- ifelse(test = rk$upper<0, yes = 0, no = rk$upper)

## ----fig.width=12, fig.height=7------------------------------------------
# convert to data.frame
rk <- as(rk, Class = "data.frame")

# create ggplot object
gg.rk <- ggplot() +
  geom_raster(
    data = rk,
    mapping = aes(x = x * 1.0e-3, y = y * 1.0e-3, fill = pred) 
  )+     
  scale_fill_continuous(
    name = "Clay content (%)",
    low = "#99CCFF",
    high = "#003333"
  ) +
  scale_x_continuous(name = "Easting (km)") +
  scale_y_continuous(name = "Northing (km)") +
  coord_equal(ratio = 1)

# plot
gg.rk

## ----fig.width = 6, fig.height=3, out.width='6cm', out.height='3cm', fig.show='hold'----
# create ggplot object
gg.lower <- ggplot() +
  geom_raster(
    data = rk,
    mapping = aes(x = x * 1.0e-3, y = y * 1.0e-3, fill = lower) 
  )+     
  scale_fill_continuous(
    name = "Clay content (%)",
    low = "#99CCFF",
    high = "#003333"
  ) +
  scale_x_continuous(name = "Easting (km)") +
  scale_y_continuous(name = "Northing (km)") +
  coord_equal(ratio = 1)

# create ggplot object
gg.upper <- ggplot() +
  geom_raster(
    data = rk,
    mapping = aes(x = x * 1.0e-3, y = y * 1.0e-3, fill = upper) 
  )+     
  scale_fill_continuous(
    name = "Clay content (%)",
    low = "#99CCFF",
    high = "#003333"
  ) +
  scale_x_continuous(name = "Easting (km)") +
  scale_y_continuous(name = "Northing (km)") +
  coord_equal(ratio = 1)

# plot
gg.lower
gg.upper

## ------------------------------------------------------------------------
ked <- krige(trend.sw, locations = d, newdata = edgeroi.grids, model = vmf, debug.level = -1)

## ------------------------------------------------------------------------
# 90% prediction interval
ked$lower <- ked$var1.pred - 1.645 * sqrt(ked$var1.var)
ked$upper <- ked$var1.pred + 1.645 * sqrt(ked$var1.var)

# correct predictions
ked$var1.pred <- ifelse(test = ked$var1.pred<0, yes = 0, no = ked$var1.pred)
ked$lower <- ifelse(test = ked$lower<0, yes = 0, no = ked$lower)
ked$upper <- ifelse(test = ked$upper<0, yes = 0, no = ked$upper)

## ------------------------------------------------------------------------
# regression-kriging
summary(rk[,-c(1:2)])
round(head(rk[,-c(1:2)]),2)

# kriging with an external drift
summary(ked@data)
round(head(ked@data),2)


###########################################################################
##### 4 Random forests modelling

## ------------------------------------------------------------------------
# convert data object to data.frame
d <- as(d, Class = "data.frame")

# copy soil property values to a new vector
z <- d$z

# copy the covariates to a new data.frame
names(d)
covar <- d[,3:14]

## ------------------------------------------------------------------------
rf <- randomForest(
  x = covar,
  y = z
)

## ------------------------------------------------------------------------
str(rf, max.level=2)

## ----out.width='5cm', out.height='5cm'-----------------------------------
varImpPlot(rf)

## ----fig.width=8, fig.height=5-------------------------------------------
# correlation plot
plot(rf$predicted, rf$y, xlab="Fitted (%)", ylab="observed (%)")
abline(0,1)

# r2 value
cor(rf$predicted,rf$y)**2

## ------------------------------------------------------------------------
edgeroi.grids <- as(edgeroi.grids, Class= "data.frame")
p.rf <- predict(rf, newdata=edgeroi.grids)

## ----out.width='4cm', out.height='4cm', fig.show='hold'------------------
# compute residual
d$resid.rf <- rf$predicted - rf$y

# check residual distribution
hist(d$resid.rf, col = "lightblue")
qqnorm(d$resid.rf)
qqline(d$resid.rf)

# residual plot
plot(rf$predicted, d$resid.rf, xlab = "Fitted (%)", ylab = "Residual (%)")
abline(0,0)

## ----out.width='5cm', out.height='5cm'-----------------------------------
# convert data.frame to a spatial object
coordinates(d) <- ~x+y

# set coordinate projection
proj4string(d) <- CRS(gda94)

# compute sample variogram.
v.rf <- variogram(d$resid.rf~1, data = d, width = 1000)
plot(v.rf)

## ----out.width='5cm', out.height='5cm'-----------------------------------
# define variogram model: initial values for psill, range, nugget based on sample variogram
vm <- vgm(psill = 30, model = "Sph", range = 5000, nugget = 80)

# fit variogram model to the experimental variogram
vmf.rf <- fit.variogram(v.rf, model = vm)

# plot variograms
plot(v.rf,vmf.rf)

## ------------------------------------------------------------------------
# inspect variogram parameters
vmf
vmf.rf

## ----results='hide'------------------------------------------------------
# convert grid data to spatial object and set projection
gridded(edgeroi.grids) <- ~x+y
proj4string(edgeroi.grids) <- CRS(gda94)

# krige the residuals
rf.k <- krige(resid.rf ~ 1, locations = d, newdata = edgeroi.grids, model = vmf.rf, debug.level = -1)

## ------------------------------------------------------------------------
rf.k$trend <- p.rf
str(rf.k@data)

## ------------------------------------------------------------------------
rf.k$pred <- rf.k$trend + rf.k$var1.pred

## ------------------------------------------------------------------------
rf.k$lower <- rf.k$pred - 1.645 * sqrt(rf.k$var1.var)
rf.k$upper <- rf.k$pred + 1.645 * sqrt(rf.k$var1.var)

## ------------------------------------------------------------------------
summary(rf.k@data)

## ------------------------------------------------------------------------
# create output folder
dir.create("./out")

# save as GeoTIFF
writeGDAL(rf.k["pred"], fname = "./out/Clay_RFK.tif", drivername = "GTiff", type = "Float32")

## ----fig.width=14, fig.height=7------------------------------------------
# convert to data.frame
rf.k <- as(rf.k, Class = "data.frame")

# create ggplot object
gg.rfk <- ggplot() +
  geom_raster(
    data = rf.k,
    mapping = aes(x = x * 1.0e-3, y = y * 1.0e-3, fill = pred) 
  )+     
  scale_fill_continuous(
    name = "Clay content (%)",
    low = "#99CCFF",
    high = "#003333"
  ) +
  scale_x_continuous(name = "Easting (km)") +
  scale_y_continuous(name = "Northing (km)") +
  coord_equal(ratio = 1)

# plot
gg.rfk


###########################################################################
##### 5 Visualizing spatial data in Google Earth and interactive maps

## ----eval=FALSE----------------------------------------------------------
# create folders to store KML files
dir.create("./out/kml_rk")
dir.create("./out/kml_rf")

# define shape of the points in the visualization
shape <- "http://maps.google.com/mapfiles/kml/pal2/icon18.png" 

# export point data
kml(d, file.name="./out/edgeroiData.kml", colour=z, balloon=TRUE, shape=shape, colour_scale=SAGA_pal[[1]], points_names=round(d$z,1))

# export regression-krigin results
gridded(rk) <- ~x+y
proj4string(rk) <- CRS(gda94)
kml(rk["pred"], file.name="clay_RK.kml",colour_scale = SAGA_pal[[1]])

# copy kml files to separate directory
system(paste("xcopy", "clay_RK.kml", shortPathName(normalizePath("./out/kml_rk"))))
system(paste("xcopy", "*.png", shortPathName(normalizePath("./out/kml_rk"))))

# export random forest model results
gridded(rf.k) <- ~x+y
proj4string(rf.k) <- CRS(gda94)
kml(rf.k["pred"], file.name="clay_RF.kml",colour_scale = SAGA_pal[[1]])

# copy kml files to separate directory
system(paste("xcopy", "clay_RF.kml", shortPathName(normalizePath("./out/kml_rf"))))
system(paste("xcopy", "*.png", shortPathName(normalizePath("./out/kml_rf"))))

# delete KML files from working directory
unlink(c("clay_RK.kml","clay_RF.kml",".png","_legend.png"))

## ----eval=FALSE----------------------------------------------------------
# convert data to WGS84 coordinate reference system
d.ll <- d
d.ll <- spTransform(d.ll, CRSobj = CRS(wgs84))

## ----eval=FALSE----------------------------------------------------------
# basic map
addMarkers(addTiles(leaflet()), data = d.ll)

# or, more elegant:
leaflet() %>%
  addTiles() %>%
  addMarkers(data = d.ll)

# creat pop-ups for markers
my_pops <- paste0(
  "<strong>Site: </strong>",
  d.ll$id,
  '<br>
  <strong> Clay content (%): </strong>',
  round(d.ll$z,1)
)

# create interactive map
leaflet() %>%
  addProviderTiles("Esri.WorldImagery") %>%
  addMarkers(data = d.ll, popup = my_pops)
# now click on a marker in the map!

## further marker enhancements
# define colour ramps
pal1 <- colorQuantile("YlOrBr", domain = d$z)
pal2 <- colorNumeric(SAGA_pal[[1]], domain = d.ll$z, na.color = "transparent")

# generate interactive maps
(ll1 <- leaflet() %>%
    addProviderTiles("Esri.WorldImagery") %>%
    addCircleMarkers(data = d.ll, color = ~pal1(z), popup = my_pops) %>%
    addLegend("bottomright", pal = pal1, values = d.ll$z,
              title = "Clay content (%) quantiles",
              opacity = 0.8)
)

# with a different color scale
(ll2 <- leaflet() %>%
    addProviderTiles("Esri.WorldImagery") %>%
    addCircleMarkers(data = d.ll, color = ~pal2(z), popup = my_pops) %>%
    addLegend("bottomright", pal = pal2, values = d.ll$z,
              title = "Clay content (%)",
              opacity = 0.8)
)

# save as HTML widget
saveWidget(ll1, file = paste0(getwd(),"/out/Edgeroi_clay_content.html",sep=""))

## ----eval=FALSE----------------------------------------------------------
# read tiff as RasterLayer
r <- raster("./out/Clay_RFK.tif")

# plot aesthetics
header <- "Clay content (%)"
pal <- colorNumeric(SAGA_pal[[1]], values(r), na.color = "transparent")

# create leaflet
(ll <- leaflet() %>% 
    addProviderTiles("Esri.WorldImagery") %>%
    addRasterImage(r, colors=pal, opacity=0.5) %>%
    addLegend(pal=pal, values=values(r), title=header)
)

# save as widget
saveWidget(ll, file = paste0(getwd(),"/out/Clay_content_RFK.html",sep=""))


###########################################################################
##### 6 Hands-on exercise

## ----results='hide'------------------------------------------------------
# fit spline
mps <- mpspline(edgeroi.spc, var.name="ORCDRC", vlow=0, vhigh=1000)

# compile a new data.frame with carbon data 
d <- data.frame(
  id = mps$idcol,
  x = edgeroi.spc@sp@coords[,1],
  y = edgeroi.spc@sp@coords[,2],
  z = mps$var.std[,2]
)

# set projection and reproject
coordinates(d) <- ~x+y
proj4string(d) <- CRS(grs80)
d <- spTransform(d, CRS(gda94))

# identify points with similar coordinates (gives problems with spatial modelling)
zerodist(d, zero = 0.0, unique.ID = FALSE)

# remove duplicates
d <- remove.duplicates(d, zero = 0.0, remove.second = TRUE)

## ----eval=FALSE----------------------------------------------------------
# overlay
dum <- over(x = ..., y = ...)

# append covariate data to 
d@data <- cbind(d@data, dum)

# convert to data.frame
d <- as(d, Class = "data.frame")

# remove observations without covariate data, clay data or coordinates
d <- na.omit(d)

# clean-up
rm(dum)

## ----eval=FALSE----------------------------------------------------------
# define trend model
names(d)
trend <- as.formula(paste("z~",paste(names(d)[3:14],collapse="+")))

# fit lm model
oc.lm <- ...

# stepwise selection
oc.slm <- ...

# inspect model
summary(oc.slm)

## ----eval=FALSE----------------------------------------------------------
# skewness statistics
skewness(...)

# histogram
hist(...)

# qqplot
qqnorm(...)
qqline(...)

# residual plot
plot(...,...)
abline(0,0)

## ----eval=FALSE----------------------------------------------------------
# log transform
d$lz <- log(d$z) # use log1p when the data contains zeros

# define trend model
names(d)
trend <- as.formula(paste("lz~",paste(names(d)[3:14],collapse="+")))

# fit lm model
loc.lm <- ...

# stepwise selection of covariates 
loc.slm <- ...

# inspect model
summary(loc.slm)

## ----eval=FALSE----------------------------------------------------------
# skewness statistics
skewness(...)

# histogram
hist(...)

# qqplot
qqnorm(...)
qqline(...)

# residual plot
plot(...,...)
abline(0,0)

## ----eval=FALSE----------------------------------------------------------
# copy residuals to data object
d$lresid <- ...

# convert data.frame to a spatial object
coordinates(d) <- ...

# set coordinate projection
proj4string(d) <- ...

# compute sample variogram (use lresid~1 as function, use width=1000)
v.oc <- ...
plot(v.oc)

# define variogram model (use a spherical model)
# choose psill and nugget values on basis of the sample variogram
vm <- ...

# fit variogram model to the experimental variogram 
vmf.oc <- ...

# plot variograms
plot(v.oc,vmf.oc)

## ----eval=FALSE----------------------------------------------------------
p.oc <- ...

## ----eval=FALSE----------------------------------------------------------
# convert to data.frame
p.oc <- as.data.frame(p.oc)

# square and sum the prediction se (se.fit) and residual sd (residual.scale)
p.oc$var <- ... + ...

## ----eval=FALSE----------------------------------------------------------
p.oc$pred <- ...

## ----eval=FALSE----------------------------------------------------------
# predicted carbon
summary(p.oc$pred)

# observed carbon
summary(d$z)

# predicted carbon (log-scale)
summary(p.oc$fit)

# observed carbon (log-scale)
summary(d$lz)

## ----eval=FALSE----------------------------------------------------------
# residual variance
summary(p.oc$residual.scale**2)

# variance of the estimated vale (model fit)
summary(p.oc$se.fit**2)

# prediction error variance
summary(p.oc$var)

## ----eval=FALSE----------------------------------------------------------
p.oc$lower <- ...
p.oc$upper <- ...


###########################################################################
##### 8 Code Solutions (hands-on exercise)

## ----eval=FALSE----------------------------------------------------------
# overlay
dum <- over(x = d, y = edgeroi.grids)

## ----eval=FALSE----------------------------------------------------------
# fit lm model
oc.lm <- lm(trend, data=d)

# stepwise selection
oc.slm <- step(oc.lm)

## ----eval=FALSE----------------------------------------------------------
# skewness statistics (e1071 package)
skewness(resid(oc.slm))

# histogram
hist(resid(oc.slm))

# qqplot
qqnorm(resid(oc.slm))
qqline(resid(oc.slm))

# residual plot
plot(oc.slm$fitted.values, resid(oc.slm), xlab = "Fitted (g/kg)", ylab = "Residual (g/kg)")

## ----eval=FALSE----------------------------------------------------------
# fit lm model
loc.lm <- lm(trend, data=d)

# stepwise selection of covariates 
loc.slm <- step(loc.lm)

## ----eval=FALSE----------------------------------------------------------
# skewness statistics (e1071 package)
skewness(resid(loc.slm))

# histogram
hist(resid(loc.slm))

# qqplot
qqnorm(resid(loc.slm))
qqline(resid(loc.slm))

# residual plot
plot(loc.slm$fitted.values, resid(loc.slm), xlab = "Fitted (g/kg)", ylab = "Residual (g/kg)")

## ----eval=FALSE----------------------------------------------------------
# copy residuals to data object
d$lresid <- resid(loc.slm)

# convert data.frame to a spatial object
coordinates(d) <- ~x+y

# set coordinate projection
proj4string(d) <- CRS(gda94)

# compute sample variogram (use lresid~1 as function, use width=1000)
v.oc <- variogram(lresid~1, data = d, width = 1000)
plot(v.oc)

# define variogram model (use a spherical model)
# choose psill and nugget values on basis of the sample variogram
vm <- vgm(psill = 0.02, model = "Sph", range = 5000, nugget = 0.13)

# fit variogram model to the experimental variogram
vmf.oc <- fit.variogram(v.oc, model = vm)

## ----eval=FALSE----------------------------------------------------------
p.oc <- predict(loc.slm, newdata = edgeroi.grids, se.fit = TRUE)

## ----eval=FALSE----------------------------------------------------------
# square and sum the prediction se (se.fit) and residual sd (residual.scale)
p.oc$var <- p.oc$se.fit**2 + p.oc$residual.scale**2

## ----eval=FALSE----------------------------------------------------------
p.oc$pred <- exp(p.oc$fit + 0.5*p.oc$var)

## ----eval=FALSE----------------------------------------------------------
p.oc$lower <- exp(p.oc$fit - 1.645 * sqrt(p.oc$var))
p.oc$upper <- exp(p.oc$fit + 1.645 * sqrt(p.oc$var))

## ----include=FALSE, cache=FALSE------------------------------------------
purl("RegressionModelling_v1.4.Rnw")
save.image("RegressionModelling_v1.4.rda")

